/***************************************************************************
 *   Copyright (C) 2008 by Dario Freddi <drf@kdemod.ath.cx>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "QueueManager.h"
#include "ProgressInterface.h"

#include <KDebug>
#include <KTempDir>

#include <QMap>

class QueueManager::Private
{
public:
    Private() : tempDir(new KTempDir()) {}

    ~Private() {}

    int generateItemId();

    QueueItem::List itemList;
    QueueItem::List::const_iterator iterator;
    QMap<int, QueueItem*> tokenActions;
    ProgressInterface *progressInterface;
    AbstractDevice::List deviceList;
    KTempDir *tempDir;
};

QueueManager::QueueManager(QObject *parent)
        : QObject(parent),
        d(new Private())
{
    d->progressInterface = new ProgressInterface(this);
    d->itemList.clear();
}

QueueManager::~QueueManager()
{
    delete d;
}

ProgressInterface *QueueManager::progressInterface()
{
    return d->progressInterface;
}

QueueItem::List QueueManager::currentQueue()
{
    return d->itemList;
}

int QueueManager::addJobToQueue(QueueItem::Action action, AbstractDevice *in, const QString &inpath,
                                AbstractDevice *out, const QString &outpath)
{
    QueueItem *item = new QueueItem;

    item->action = action;
    item->in_device = in;
    item->in_path = inpath;
    item->out_device = out;
    item->out_path = outpath;
    item->jobId = d->generateItemId();

    d->itemList.append(item);

    kDebug() << "Added to queue: copy" << inpath << "to" << outpath << ", job ID: " << item->jobId;

    return item->jobId;
}

int QueueManager::Private::generateItemId()
{
    int new_id = 0;

    foreach(QueueItem *itm, itemList) {
        if (itm->jobId >= new_id) {
            new_id = itm->jobId + 1;
        }
    }

    return new_id;
}

void QueueManager::removeJobFromQueue(int jobId)
{
    foreach(QueueItem *itm, d->itemList) {
        if (itm->jobId == jobId) {
            d->itemList.removeOne(itm);
            delete itm;
        }
    }
}

void QueueManager::clearQueue()
{
    while (!d->itemList.isEmpty()) {
        QueueItem *itm = d->itemList.takeFirst();
        delete itm;
    }

    d->itemList.clear();

    emit queueCleared();
}

void QueueManager::processQueue()
{
    d->iterator = d->itemList.constBegin();

    progressInterface()->reset();
    progressInterface()->setItems(d->itemList);

    processNextQueueItem();
}

void QueueManager::processNextQueueItem()
{
    if (d->iterator == d->itemList.constEnd() || !(*d->iterator)) {
        kDebug() << "Queue ended";
        clearQueue();

        // Let's reload needed models
        foreach(AbstractDevice *ent, d->deviceList) {
            ent->reloadModel();
        }

        d->deviceList.clear();

        progressInterface()->queueCompleted();
        return;
    }

    kDebug() << "Processing next queue item";

    progressInterface()->setCurrentItem(*d->iterator);

    int token;

    switch ((*d->iterator)->action) {
    case QueueItem::Copy:
    case QueueItem::Move:
        progressInterface()->setCurrentItemAction(ProgressInterface::Getting);

        connect((*d->iterator)->in_device, SIGNAL(fileCopiedFromDevice(int, const QString&)),
                this, SLOT(fileCopiedFromDevice(int, const QString&)));
        connect((*d->iterator)->in_device, SIGNAL(actionProgressChanged(int)),
                progressInterface(), SLOT(setCurrentItemProgress(int)));

        token = (*d->iterator)->in_device->getFileFromDevice((*d->iterator)->in_path, d->tempDir->name());

        d->tokenActions[token] = (*d->iterator);

        break;

    case QueueItem::Delete:
        progressInterface()->setCurrentItemAction(ProgressInterface::Deleting);

        connect((*d->iterator)->in_device, SIGNAL(pathRemovedFromDevice(int, const QString&)),
                this, SLOT(pathRemovedFromDevice(int, const QString&)));

        token = (*d->iterator)->in_device->removePath((*d->iterator)->in_path);

        d->tokenActions[token] = (*d->iterator);

        if (!d->deviceList.contains((*d->iterator)->in_device)) {
            d->deviceList.append((*d->iterator)->in_device);
        }

        break;

    default:
        break;
    }

}

void QueueManager::fileCopiedFromDevice(int token, const QString &filePath)
{
    int new_token;

    switch (d->tokenActions[token]->action) {
    case QueueItem::Copy:
        progressInterface()->setCurrentItemAction(ProgressInterface::Sending);
        disconnect(d->tokenActions[token]->in_device, SIGNAL(fileCopiedFromDevice(int, const QString&)),
                   this, SLOT(fileCopiedFromDevice(int, const QString&)));
        disconnect((*d->iterator)->in_device, SIGNAL(actionProgressChanged(int)),
                   progressInterface(), SLOT(setCurrentItemProgress(int)));

        kDebug() << "Copying " << filePath << d->tokenActions[token]->out_path;

        new_token = d->tokenActions[token]->out_device->sendFileToDevice(filePath, d->tokenActions[token]->out_path);

        d->tokenActions[new_token] = d->tokenActions[token];

        connect(d->tokenActions[new_token]->out_device, SIGNAL(fileCopiedToDevice(int, const QString&)),
                this, SLOT(fileCopiedToDevice(int, const QString&)));

        if (!d->deviceList.contains(d->tokenActions[new_token]->out_device)) {
            d->deviceList.append(d->tokenActions[new_token]->out_device);
        }

        break;
    case QueueItem::Move:
        progressInterface()->setCurrentItemAction(ProgressInterface::Sending);
        disconnect(d->tokenActions[token]->in_device, SIGNAL(fileCopiedFromDevice(int, const QString&)),
                   this, SLOT(fileCopiedFromDevice(int, const QString&)));

        new_token = d->tokenActions[token]->out_device->sendFileToDevice(filePath, d->tokenActions[token]->out_path);

        d->tokenActions[new_token] = d->tokenActions[token];

        connect(d->tokenActions[new_token]->out_device, SIGNAL(fileCopiedToDevice(int, const QString&)),
                this, SLOT(fileCopiedToDevice(int, const QString&)));

        if (!d->deviceList.contains(d->tokenActions[new_token]->out_device)) {
            d->deviceList.append(d->tokenActions[new_token]->out_device);
        }

        break;
    default:
        break;
    }
}

void QueueManager::fileCopiedToDevice(int token, const QString &/*filePath*/)
{
    disconnect(d->tokenActions[token]->out_device, SIGNAL(fileCopiedToDevice(int, const QString&)),
               this, SLOT(fileCopiedToDevice(int, const QString&)));

    int new_token;

    switch (d->tokenActions[token]->action) {
    case QueueItem::Copy:
        kDebug() << "Copy completed!!";
        d->iterator += 1;
        processNextQueueItem();
        break;

    case QueueItem::Move:
        progressInterface()->setCurrentItemAction(ProgressInterface::Deleting);
        connect((*d->iterator)->in_device, SIGNAL(pathRemovedFromDevice(int, const QString&)),
                this, SLOT(pathRemovedFromDevice(int, const QString&)));

        new_token = d->tokenActions[token]->in_device->removePath(d->tokenActions[token]->in_path);

        d->tokenActions[new_token] = d->tokenActions[token];

        if (!d->deviceList.contains(d->tokenActions[token]->in_device)) {
            d->deviceList.append(d->tokenActions[token]->in_device);
        }
        break;

    default:
        break;
    }
}

void QueueManager::pathRemovedFromDevice(int token, const QString &)
{
    disconnect(d->tokenActions[token]->in_device, SIGNAL(pathRemovedFromDevice(int, const QString&)),
               this, SLOT(pathRemovedFromDevice(int, const QString&)));

    kDebug() << "Item deleted!!";
    d->iterator += 1;
    processNextQueueItem();
}

#include "QueueManager.moc"
